/*
 Kraft-Weg Messung
 Der Vorschub einer Mutter auf einer Gewindespindel wird durch Erfassen des Drehwinkels ermittelt.
 Alle 0,1 Inch wird der Vorschub aktualisiert und ein Analogsignal aus z.B einer Kraftmessung eingelesen.
 Ein zweites Puls Signal vom Encoder ermöglichet die Bestimmung der Drehrichtung

 für die Kraft Messung:
 Arduino Digital pin 2 -> HX711 CLK
 Arduino Digital pin 3 -> DAT
 5V -> VCC
 GND -> GND
 */

#include <SD.h>
#include <SPI.h>
#include <HX711.h>

#define calibration_factor -31000.0 //This value is obtained using the SparkFun_HX711_Calibration sketch
#define DOUT  4
#define CLK  5

HX711 scale(DOUT, CLK);

// this constant won't change:
const int  A = 2;    // Signal von Kanal A auf digital input 2
const int  B = 3;    // Signal von Kanal B auf digital input 3
const int NPuls = 600;  // Anzahl der Impulse je Umdrehung
const int Ninch = 1524;  // Anzahl der Pulse für 0,4" bei Gewindesteigung von 4mm 
const float dX=0.4;   //increment für X bei erreichen von Ninch


const int chipSelect = 10; // für SD Karte

// Variables will change:
int PCount = 0;         // counter for the number of pulses
int StateA = 0;         // Signal Level von A
int StateB = 0;         // Signal Level von B
int StatInt=0;          // wird in ISR auf 1 gesetzt
float XStart=0;         // Standhöhein mm. Wird später nach Abfrage eingegeben
float X=0;              // aktuelle Auszugslänge in Zoll ausgegeben
float Kraft=0;           // Auszugskraft in lbs ausgegeben


void setup() 
{
  pinMode(A, INPUT_PULLUP);  //Signal A ist Digital Input on interrupt 0(pin2) with Pullup
  pinMode(B, INPUT_PULLUP);  //Signal B ist Digital Input on interrupt 0 (pin3) with Pullup  
  pinMode(10, OUTPUT);//for Data logger
  pinMode(13, OUTPUT);//LED
 
  
  scale.set_scale(calibration_factor); //This value is obtained by using the SparkFun_HX711_Calibration sketch
  scale.tare();  //Assuming there is no weight on the scale at start up, reset the scale to 0

  // see if the card is present and can be initialized:
  if (!SD.begin(chipSelect)) 
  {
    return;// don't do anything more:
  }
      XStart=0;  // Standhöhe in mm. Kann später durch eine Einhabe mit Poti oder Taster direkt eingegeben werden
      X=XStart/25.4;//Umrechnung mm in inch
   // open the file. 
      File dataFile = SD.open("KraftWeg.txt", FILE_WRITE);
      // if the file is available, write to it:
      if (dataFile) 
      {
        dataFile.print(X);    //schreibt hier die Standhöhe in die Datei, Kraft ist ca. 0
        dataFile.print(";");
        dataFile.println(Kraft);
        dataFile.close();
      }     
  attachInterrupt(0,countA, RISING);  // Interrupt 0 (an Pin2)
}

void loop() 
{  
   if (StatInt==1)
   {
    PCount++;
    if(PCount>=Ninch)
      {
      //digitalWrite(13,HIGH); //LED an
      StateB= digitalRead(B);  // Signal B einlesen: 
      if(StateB==HIGH)
        {
        X=X-dX;      // Auszugslänge um dX" vermindern
        }
      else
        {
        X=X+dX;     // Auszugslänge um dX" erhöhen
        }
      Kraft=scale.get_units(),2; //Kraft von Messverstärker auslesen
      // open the file. 
      File dataFile = SD.open("KraftWeg.txt", FILE_WRITE);
      // if the file is available, write to it:
      if (dataFile) 
      {
        dataFile.print(X);
        dataFile.print(";");
        dataFile.println(Kraft);
        dataFile.close();
      }
      //digitalWrite(13,LOW);//LED aus
      PCount=0;  // reset pulse counter
    }
    StatInt=0;
   }
}  


void countA()
  {
  StatInt=1;//Flag setzen, dass ein Interrupt durch Puls Flanke war
  }





